///////////////////////////////////////////////////////
// sq80dump - dump complete SQ80 disks               //
// written 1999 by Rainer Buchty (rainer@buchty.net) //
///////////////////////////////////////////////////////

#include <fcntl.h>
#include <linux/fd.h>
#include <stdio.h>
#include "sq80.h"

#define FROM_SQ 0
#define TO_SQ	1

void usage(void){
	printf("sq80dump -if=file -of=file\n");
	printf("\tParameters:\n");
	printf("\t-if\tinput file\n");
	printf("\t-of\toutput file\n");
	exit(0);
}

void error(const char *message){
	printf("%s\n",message);
	exit(-1);
}

void sqdisk_seek(int dnum, int c, int h) {
char dummy[256];
	sprintf(dummy,"fdrawcmd seek %i %i 2>/dev/null",(dnum&3)|(h<<2),c);
	system(dummy);
}

int translate_chs(int track,int head, int sector) {
int ptr=0;
        if (track>0) ptr=track*11264;                           /* complete track offset */
        if (head==1) ptr+=5632;                                 /* "half" track offset */
        if ((sector>0) && (sector!=5)) ptr+=sector*1024;        /* big sector offset */
        if (sector==5) ptr+=5120;                               /* small sector offset */
        return ptr+10;						/* header size offset */
}

int main(int argc, char **argv){
char inname[256],outname[256],dummy[256],buffer[1024];
FILE *fd;

int dnum=0, i, dir=-1, size, sizecode;
int c,h,s;

	argv++; argc--;

	if (argc!=2) usage();
	
	while(*argv!=NULL){
		if (strncmp(*argv,"-if=",4)==0) strcpy(inname,*argv+4);
		else if (strncmp(*argv,"-of=",4)==0) strcpy(outname,*argv+4);
		else usage();
		argv++;
	}

	for(i=0;i<4;i++) {
		if (strncmp(inname,floppy[i],8)==0) {
			dnum=i;
			dir=FROM_SQ;
		}
		if (strncmp(outname,floppy[i],8)==0) {
			dnum=i;
			dir=TO_SQ;
		}
	}
	if (dir==-1) usage();

	/* reset floppy */
	if ((i=open(floppy[dnum],O_NDELAY))<0) error("Cant access floppy drive.\n");
	if (ioctl(i,FDRESET,2)<0) {
		close(i);
		error("Cant reset floppy drive.");
	}
	close(i);

	/* tune to SQ80 disk */
	sprintf(dummy,"fdrawcmd readid %i rate=2 2>/dev/null",dnum);
	system(dummy);

	sprintf(dummy,"fdrawcmd recalibrate %i 2>/dev/null",dnum);
	system(dummy);
	

	if (dir==FROM_SQ) {
		printf("Dumping SQ80 disk in drive %i to file \"%s\"\n",dnum,outname);
		sprintf(dummy,"printf !SQ80DISK! > %s",outname);
		system(dummy);
	} else printf("Dumping file \"%s\" to SQ80 disk in drive %i.\n",inname,dnum);

	fflush(stdout);

	c=0; h=0; s=0; 
	while (c!=80) {
		if (s==0) sqdisk_seek(dnum,c,h);
		if (s==5) {
			size=512;
			sizecode=2;
		} else {
			size=1024;
			sizecode=3;
		}

		printf("Accessing track %2i, head %i, sector %i (%4i bytes)\r",c,h,s,size);
		fflush(stdout);

		if (dir==FROM_SQ) {
			sprintf(dummy,"fdrawcmd read %i %i %i %i %i 0 0x1b 0xff length=%i rate=2 2>/dev/null >> %s",(dnum&3)|(h<<2),c,h,s,sizecode,size,outname);	
			fd=popen(dummy,"r");
				fread(buffer,1,size,fd);
			fclose(fd);
		} else {
			fd=fopen(inname,"r");
				i=translate_chs(c,h,s);
				fseek(fd,i,SEEK_SET);
				fread(buffer,1,size,fd);
			fclose(fd);
			sprintf(dummy,"fdrawcmd write %i %i %i %i %i 0 0x1b 0xff length=%i rate=2 2>/dev/null",(dnum&3)|(h<<2),c,h,s,sizecode,size,inname);	
			fd=popen(dummy,"w");
				fwrite(buffer,1,size,fd);
			fclose(fd);
		}
		if (s<5) s++;
		else {
			h++;
			s=0;
		}
		if (h>1) {
			h=0;
			c++;
		}
	}
	printf("\n");
	exit(0);
}
